#include <xc.h>
#include "pic_init.h"

#define PIN_DIR_UART_RX				(TRISAbits.TRISA0)	// pin 2
#define PIN_DIR_UART_TX				(TRISAbits.TRISA1)	// pin 3
#define PIN_DIR_PIN_I2C_SCL			(TRISBbits.TRISB4)	// pin 25
#define PIN_DIR_PIN_I2C_SDA			(TRISBbits.TRISB5)	// pin 26
#define ENABLE 1
#define DISABLE 0
#define INPUT 1
#define OUTPUT 0

static void pic_pin_directions(void);
static void pic_pin_default(void);
static void pic_pic_pps(void);
static void pic_uart2_init(void);
static void pic_set_interrupts(void);
static void pic_ic2_module1_init(void);
static void pic_pps_unlock(void);
static void pic_pps_lock(void);

static void pic_ic2_module1_init(void)
{
	// standard slew rate for 100kHz
	SSP1STATbits.SMP = 1;		

	// Master, clock = Fosc / (4 * (SSP1ADD + 1))
	SSP1CON1bits.SSPM = 0x8;	

	// set up I2C transaction parameters
	SSP1CON2 = 0x00;			

	// set baud rate to 100kHz
	SSP1ADD = 0x13;
}

static void pic_uart2_init(void)
{
	
	RCSTA2bits.SPEN = 1;	// enable the module

	TXSTA2bits.SYNC = 0;	// Asynchronous
	TXSTA2bits.BRGH = 0;	// High Baud Rate Select bit set to Low speed
	BAUDCON2bits.BRG16 = 0;	// 8 bit generator

	SPBRG2 = 12;

	RCSTA2bits.RX9 = 0; 	// 8 bit reception mode
	RCSTA2bits.CREN = 1;	// enable the continuous receiver

	TXSTA2bits.TX9 = 0;		// 8 bits transmission mode
	TXSTA2bits.TXEN = 1;	// enable the transmission
}

static void pic_pin_default(void)
{
	/* UART (module 2) */
	ANCON0bits.PCFG0 = 1;	// AN0 to digital
	ANCON0bits.PCFG1 = 1;	// AN1 to digital
}

static void pic_pin_directions(void)
{
	/* I2C (module 1) */
	PIN_DIR_PIN_I2C_SCL = INPUT;
	PIN_DIR_PIN_I2C_SDA = INPUT;

	/* UART (module 2) */
	PIN_DIR_UART_RX = INPUT;
	PIN_DIR_UART_TX = OUTPUT;
}

static void pic_pps_unlock(void)
{
	// unlock registers
	INTCONbits.GIE = 0;
	EECON2 = 0x55;
	EECON2 = 0xAA;
	PPSCONbits.IOLOCK = 0;
}

static void pic_pps_lock(void)
{
	// lock registers
	EECON2 = 0x55;
	EECON2 = 0xAA;
	INTCONbits.GIE = 1;
	PPSCONbits.IOLOCK = 1;
}

static void pic_pic_pps(void)
{
	pic_pps_unlock();
   
	// UART2
	RPINR16 = 0x00; 	// RA0/RP0	->EUSART2:RX2
	RPOR1 = 0x06; 		// RA1/RP1	 = EUSART2:TX2

	pic_pps_lock();
}

static void pic_set_interrupts(void)
{
	INTCONbits.GIE = DISABLE;	// Disable general interrupts
	RCONbits.IPEN = DISABLE;	// Disable interrupt priorities

	/* Clear all interrupts and flags */
	PIE1 = 0;
	PIE2 = 0;
	PIE3 = 0;
	PIE4 = 0;
	PIE5 = 0;
	
	PIR1 = 0;
	PIR2 = 0;
	PIR3 = 0;
	PIR4 = 0;
	PIR5 = 0;

	/* CONFIGURATIONS */
	RCONbits.IPEN = DISABLE;	// Disable interrupt priorities
	INTCONbits.PEIE = ENABLE;	// Enable Peripheral interrupts
	INTCONbits.GIE = ENABLE;	// Enable general interrupts
}

void putch(char data)
{
	while (!TX2IF);
	TXREG2 = data; 
}

void pic_init(void)
{
	// set internal oscillator to 8MHz
	OSCCONbits.IRCF = 7;

	// system clock select bits
	OSCCONbits.SCS = 0;

	pic_pin_directions();
	pic_pin_default();
	pic_pic_pps();
	pic_uart2_init();  
	pic_set_interrupts();
	pic_ic2_module1_init();
}